<?php

namespace Drupal\Tests\commerce_worldline\Functional;

use Drupal\commerce_payment\Entity\PaymentGateway;
use Drupal\Tests\commerce\Functional\CommerceBrowserTestBase;

/**
 * Tests the admin payment UI.
 *
 * @group commerce_worldline
 */
class PaymentAdminTest extends CommerceBrowserTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'commerce_worldline',
  ];

  /**
   * {@inheritdoc}
   */
  protected function getAdministratorPermissions() {
    return array_merge([
      'administer commerce_payment_gateway',
    ], parent::getAdministratorPermissions());
  }

  /**
   * Tests creating a payment gateway.
   */
  public function testPaymentGatewayCreation() {
    $this->drupalGet('admin/commerce/config/payment-gateways');
    $this->getSession()->getPage()->clickLink('Add payment gateway');
    $this->assertSession()->addressEquals('admin/commerce/config/payment-gateways/add');

    $values = [
      'label' => 'SIPS',
      'plugin' => 'sips_payment',
      'status' => '1',
      'id' => 'example',
      'configuration[sips_interface_version]' => 'llama',
      'configuration[sips_passphrase]' => 'kitten',
      'configuration[sips_merchant_id]' => 'giraffe',
      'configuration[sips_key_version]' => 'owl',
    ];
    $this->submitForm($values, 'Save');
    $this->assertSession()->addressEquals('admin/commerce/config/payment-gateways');
    $this->assertSession()->responseContains('SIPS');
    $this->assertSession()->responseContains('Test');

    $payment_gateway = PaymentGateway::load('example');
    $this->assertEquals('example', $payment_gateway->id());
    $this->assertEquals('SIPS', $payment_gateway->label());
    $this->assertEquals('sips_payment', $payment_gateway->getPluginId());
    $this->assertTrue($payment_gateway->status());
    $payment_gateway_plugin = $payment_gateway->getPlugin();
    $this->assertEquals('0', $payment_gateway_plugin->getMode());

    $configuration = $payment_gateway_plugin->getConfiguration();
    $this->assertEquals('llama', $configuration['sips_interface_version']);
    $this->assertEquals('kitten', $configuration['sips_passphrase']);
    $this->assertEquals('giraffe', $configuration['sips_merchant_id']);
    $this->assertEquals('owl', $configuration['sips_key_version']);
  }

}
